#include "mbed.h"
#include "rtos.h"
#include "st7565LCD.h"

#define SPI_SPEED   (4000000)
#define SPI_HEADER  (0x55)

BusIn Switches(PA_0, PA_1, PA_4, PB_0, PC_1, PC_0);

SPI SpiM(PA_7, PA_6, PA_5); // mosi, miso, sclk
DigitalOut SpiMCs(PB_6);

InterruptIn stepChangeInterrupt(PC_7);

//ST7565(PinName mosi, PinName sclk, PinName cs, PinName rst, PinName a0);
ST7565 gLCD(PB_15, PB_13, PB_12, PB_2, PB_1);

volatile bool isStepChanged = false;
uint8_t prevSendVal = 0x00;

void setChangeStep()
{
    isStepChanged = true;
}

int main()
{
    printf("\r\n\nNucleo rtos SPI Master Test..\r\n");
    
    #if DEVICE_SPI_ASYNCH
    printf("DEVICE_SPI_ASYNCH: defined\r\n");
    #else
    printf("DEVICE_SPI_ASYNCH: not defined\r\n");
    #endif
    
    // Setup LCD
    
    gLCD.begin(0x10);
    gLCD.drawstring(0, 0, "SPI Master Test");
    gLCD.display();
        
    // Setup Switches
    Switches.mode(PullUp);
    /*
    while(1) {
        printf("%x\r\n", ~Switches.read() &0x3f);
        Thread::wait(100);
    }
    */
    
    // Setup Interrupt
    stepChangeInterrupt.fall(&setChangeStep);
    
    // Setup SPI
    SpiMCs = 1;
    SpiM.format(16, 0);
    SpiM.frequency(SPI_SPEED);
    
    Thread::wait(1000);
    //gLCD.clear();
    
    uint8_t count = 0;
    for (;;) {
        uint16_t sendVal = (SPI_HEADER << 8) | count;
        SpiMCs = 0;
        uint16_t receivedVal = SpiM.write(sendVal);
        SpiMCs = 1;
        count++;
        if (isStepChanged) {
            char lineBuffer[20];
            sprintf(lineBuffer, "Step: %02d", receivedVal);
            gLCD.drawstring(0, 0, lineBuffer);
            gLCD.display();
            isStepChanged = false;

        }
        Thread::wait(1);
    }
}
